#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# DNS::Resolver module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 20;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'DNS::Resolver';

        use_ok('Tester');
        use_ok($TESTED_CLASS);

        $ENV{PATH} = '/bin:/usr/bin';
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $DNS::Resolver::VERSION

### Attempting to create DNS::Resolver instance...
my $instance = $TESTED_CLASS->new();
isa_ok($instance, $TESTED_CLASS);

### Viewing the created instance: $instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

is($instance->resolve_ip('195.113.144.230'), 'www.cesnet.cz',  "Resolve IP test 01");
is($instance->resolve_ip('195.113.144.234'), 'mail.cesnet.cz', "Resolve IP test 02");
is($instance->resolve_ip('195.113.144.230'), 'www.cesnet.cz',  "Resolve IP test 03");
is($instance->resolve_ip('195.113.144.234'), 'mail.cesnet.cz', "Resolve IP test 04");

is($instance->resolve_ip('1.2.3.4'), '', "Resolve IP test 05");
is($instance->resolve_ip('1.2.3.4'), '', "Resolve IP test 06");

is($instance->resolve_host('www.cesnet.cz'),  '195.113.144.230', "Resolve hostname test 01");
is($instance->resolve_host('mail.cesnet.cz'), '195.113.144.234', "Resolve hostname test 02");
is($instance->resolve_host('www.cesnet.cz'),  '195.113.144.230', "Resolve hostname test 03");
is($instance->resolve_host('mail.cesnet.cz'), '195.113.144.234', "Resolve hostname test 04");

is($instance->resolve_host('xxx.cesnet.cz'), '', "Resolve hostname test 05");
is($instance->resolve_host('xxx.cesnet.cz'), '', "Resolve hostname test 06");

is_deeply([$instance->resolve('195.113.144.230', undef)], ['195.113.144.230','www.cesnet.cz'], "Resolve test 01");
is_deeply([$instance->resolve(undef, 'www.cesnet.cz')],   ['195.113.144.230','www.cesnet.cz'], "Resolve test 02");

is_deeply([$instance->resolve('195.113.144.430', undef)], ['195.113.144.430', ''],   "Resolve test 03");
is_deeply([$instance->resolve(undef, 'wwwww.cesnet.cz')],   ['', 'wwwww.cesnet.cz'], "Resolve test 04");

my $exstats = {
        'hit' => 8,
        'miss' => 8,
        'cached' => 8,
        'resolved_ip' => 2,
        'resolved_host' => 2,
        'unresolved' => 4,
    };
diag(Dumper($instance->stats())) if Tester::D();
is_deeply($instance->stats(), $exstats, "Stats test 01");
