#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Message::Uniter module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 27;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Message::Uniter';

        use_ok('Tester');
        use_ok('Mentat::Message::Builder::IDEA');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
### $Mentat::Message::Builder::IDEA::VERSION
### $Mentat::Message::Uniter::VERSION

#-------------------------------------------------------------------------------

# Prepare builder configurations
my $builder_config = {
        'analyzer_id'       => 'an-id',
        'analyzer_ver'      => '1.0.0',
        'analyzer_class'    => 'an-class',
        'analyzer_os'       => 'an-os',
        'analyzer_osver'    => 'an-osver',
        'analyzer_location' => 'hq',
        'analyzer_host'     => 'host',
        'analyzer_ipv4'     => '127.0.0.1',
        'analyzer_ipv6'     => 'fe80::1',
    };

# Prepare message configurations
my $message_config1 = {
        'detect_time' => 1356955200,
    };
my $message_config2 = {
        'detect_time' => 1356955800,
    };

# IDEA message creation using rules
my $mpath_rules1 = [
        ['Alert/@name',             'sshd1'],
        ['Alert/counter',           '500'],
        ['Alert/duration',          '500'],
        ['Alert/othercounter',      '500'],
        ['Alert/yetanothercounter', '500'],
        ['Alert/thiscounter',       '500'],
        ['Alert/thatcounter',       '500'],
        ['Alert/address',           '195.113.144.194'],
    ];
my $mpath_rules2 = [
        ['Alert/@name',             'sshd2'],
        ['Alert/counter',           '100'],
        ['Alert/duration',          '100'],
        ['Alert/othercounter',      '100'],
        ['Alert/yetanothercounter', '100'],
        ['Alert/thiscounter',       '100'],
        ['Alert/thatcounter',       '100'],
        ['Alert/address',           '192.115.144.194'],
    ];

# Aggregation ruleset
my $RULESET = [
        ['sum',  'Alert/duration'],
        ['sub',  'Alert/othercounter'],
        ['mul',  'Alert/yetanothercounter'],
        ['div',  'Alert/thatcounter'],
        ['mod',  'Alert/thiscounter'],
        ['con',  'Alert/@name'],
        ['min',  'DetectTime', 'Aggr/DetectTime/min'],
        ['max',  'DetectTime', 'Aggr/DetectTime/max'],
        ['inc',  'Alert/counter'],
        ['join', 'Alert/address'],
    ];

#-------------------------------------------------------------------------------

# Now attempt to create a class instance and check it
my $builder = Mentat::Message::Builder::IDEA->new($builder_config);
### $builder

my $message1 = $builder->build(rules     => $mpath_rules1,
                               name_args => $message_config1);
### $message1
my $message2 = $builder->build(rules     => $mpath_rules2,
                               name_args => $message_config2);
### $message2

### Attempting to create object instance
#my $instance = $TESTED_CLASS->new($RULESET);
my $instance = $TESTED_CLASS->new('./conf/aggregation-rules.conf');
isa_ok($instance, $TESTED_CLASS);

diag("Instance dump 01: " . Dumper($instance)) if Tester::D();

#-------------------------------------------------------------------------------

is($instance->_unite('sum', 5, 6), 11,     "_unite test 01");
is($instance->_unite('sub', 5, 3),  2,     "_unite test 02");
is($instance->_unite('mul', 2, 3),  6,     "_unite test 03");
is($instance->_unite('div', 6, 3),  2,     "_unite test 04");
is($instance->_unite('mod', 7, 2),  1,     "_unite test 05");
is($instance->_unite('con', 7, 2), '7, 2', "_unite test 06");
is($instance->_unite('min', 7, 2),  2,     "_unite test 07");
is($instance->_unite('max', 7, 2),  7,     "_unite test 08");
is($instance->_unite('inc', 7, 2),  8,     "_unite test 09");
is($instance->_unite('frst', 7, 2), 7,     "_unite test 10");
is($instance->_unite('last', 7, 2), 2,     "_unite test 11");
is_deeply($instance->_unite('join', [1,2,3], [1,2,3]), [1,1,2,2,3,3], "_unite test 12");
is_deeply($instance->_unite('ujoin',[1,2,3], [1,2,3]), [1,2,3],       "_unite test 13");

#-------------------------------------------------------------------------------

my $result = $instance->unite($RULESET, [$message1, $message2]);
diag("Result dump 01: " . Dumper($result)) if Tester::D();

is($result->path_value('Alert/@name'),             'sshd1, sshd2',         "Check test 01");
is($result->path_value('Alert/counter'),           501,                    "Check test 02");
is($result->path_value('Alert/duration'),          600,                    "Check test 03");
is($result->path_value('Alert/othercounter'),      400,                    "Check test 04");
is($result->path_value('Alert/yetanothercounter'), 50000,                  "Check test 05");
is($result->path_value('Alert/thiscounter'),       0,                      "Check test 06");
is($result->path_value('Alert/thatcounter'),       5,                      "Check test 07");
is("".$result->path_value('Aggr/DetectTime/min'), '2012-12-31 12:00:00Z', "Check test 08");
is("".$result->path_value('Aggr/DetectTime/max'), '2012-12-31 12:10:00Z', "Check test 09");
is_deeply($result->path_values('Alert/address'),   ['192.115.144.194','195.113.144.194'], "Check test 10");
