#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Stats::RRD module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 60145;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Stats::RRD';

        use_ok('Tester');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Mentat::Stats::RRD::VERSION) if Tester::D();

my $WORK_DIR = './spool';
my @TEST_FILES = ('proc.a01', 'proc.a02', 'proc._totals', 'rep.a01', 'rep.a02', 'rep._totals');
my @TEST_GRAPHS = qw(
        proc.l6hours.png proc.l24hours.png proc.lweek.png proc.l4weeks.png proc.l3months.png proc.l6months.png proc.l12months.png
        proc.l6hours-t.png proc.l24hours-t.png proc.lweek-t.png proc.l4weeks-t.png proc.l3months-t.png proc.l6months-t.png proc.l12months-t.png

        proc.l6hours.spark.png proc.l24hours.spark.png proc.lweek.spark.png proc.l4weeks.spark.png
        proc.l6hours-t.spark.png proc.l24hours-t.spark.png proc.lweek-t.spark.png proc.l4weeks-t.spark.png

        proc.l6hours.xport.json proc.l24hours.xport.json proc.lweek.xport.json proc.l4weeks.xport.json proc.l3months.xport.json proc.l6months.xport.json proc.l12months.xport.json
        proc.l6hours-t.xport.json proc.l24hours-t.xport.json proc.lweek-t.xport.json proc.l4weeks-t.xport.json proc.l3months-t.xport.json proc.l6months-t.xport.json proc.l12months-t.xport.json

        rep.l6hours.png rep.l24hours.png rep.lweek.png rep.l4weeks.png rep.l3months.png rep.l6months.png rep.l12months.png
        rep.l6hours-t.png rep.l24hours-t.png rep.lweek-t.png rep.l4weeks-t.png rep.l3months-t.png rep.l6months-t.png rep.l12months-t.png

        rep.l6hours.spark.png rep.l24hours.spark.png rep.lweek.spark.png rep.l4weeks.spark.png
        rep.l6hours-t.spark.png rep.l24hours-t.spark.png rep.lweek-t.spark.png rep.l4weeks-t.spark.png

        rep.l6hours.xport.json rep.l24hours.xport.json rep.lweek.xport.json rep.l4weeks.xport.json rep.l3months.xport.json rep.l6months.xport.json rep.l12months.xport.json
        rep.l6hours-t.xport.json rep.l24hours-t.xport.json rep.lweek-t.xport.json rep.l4weeks-t.xport.json rep.l3months-t.xport.json rep.l6months-t.xport.json rep.l12months-t.xport.json
    );
my $TS = time;
my $MAX = $TS;

my $R1 = {'proc' => [['proc._totals','proc','_totals','./spool/proc._totals.rrd'],
                     ['proc.a01','proc','a01','./spool/proc.a01.rrd'],
                     ['proc.a02','proc','a02','./spool/proc.a02.rrd']],
          'rep' => [['rep._totals','rep','_totals','./spool/rep._totals.rrd'],
                    ['rep.a01','rep','a01','./spool/rep.a01.rrd'],
                    ['rep.a02','rep','a02','./spool/rep.a02.rrd']]};
my $R2 = {'rep' => [['rep._totals','rep','_totals','./spool/rep._totals.rrd'],
                    ['rep.a01','rep','a01','./spool/rep.a01.rrd'],
                    ['rep.a02','rep','a02','./spool/rep.a02.rrd']]};
my $R3 = {'proc' => [['proc._totals','proc','_totals','./spool/proc._totals.rrd'],
                     ['proc.a01','proc','a01','./spool/proc.a01.rrd']],
          'rep' => [['rep._totals','rep','_totals','./spool/rep._totals.rrd'],
                    ['rep.a01','rep','a01','./spool/rep.a01.rrd']]};
my $R4 = {'rep' => [['rep._totals','rep','_totals','./spool/rep._totals.rrd'],
                    ['rep.a01','rep','a01','./spool/rep.a01.rrd']]};

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new(reports_dir => $WORK_DIR, rrds_dir => $WORK_DIR, titles => {'proc' => 'Events processed','rep' => 'Reports processed'});
isa_ok($instance, $TESTED_CLASS);

foreach my $f (@TEST_FILES) {
    ok($instance->prepare_db($f, time), 'Prepare DB test 01');
}

is_deeply($instance->find_dbs(),            $R1, "Find test 01");
is_deeply($instance->find_dbs('rep'),       $R2, "Find test 02");
is_deeply($instance->find_dbs(undef, '02'), $R3, "Find test 03");
is_deeply($instance->find_dbs('rep','02'),  $R4, "Find test 04");

eval {
    is($instance->update_all(int(rand(1000)), $TS),                   6, "Update_all test 01");
    is($instance->update_all(int(rand(1000)), $TS+300),               6, "Update_all test 02");
    is($instance->update_all(int(rand(1000)), $TS+600,  'rep'),       3, "Update_all test 03");
    is($instance->update_all(int(rand(1000)), $TS+900,  'rep'),       3, "Update_all test 04");
    is($instance->update_all(int(rand(1000)), $TS+1200, 'rep', '02'), 2, "Update_all test 05");
    is($instance->update_all(int(rand(1000)), $TS+1500, 'rep', '02'), 2, "Update_all test 06");

    my $tsc;
    foreach my $f (@TEST_FILES) {
        foreach my $i (6..10000) {
            $tsc = $TS + ($i*300);
            $MAX = $tsc if $tsc > $MAX;
            is($instance->update($f, int(rand(1000)), $tsc), 1, "Update test $f $tsc");
        }
    }

    $instance->generate_graphs($MAX);
};
if ($@) {
    fail($@);
}

foreach my $f (@TEST_FILES) {
    is(-f "$WORK_DIR/$f.rrd", 1, "Database presence test $f");
    is(unlink("$WORK_DIR/$f.rrd"), 1, "Remove database test $f");
}

foreach my $f (@TEST_GRAPHS) {
    is(-f "$WORK_DIR/$f", 1, "Graph presence test $f");
    is(unlink("$WORK_DIR/$f"), 1, "Remove graph test $f") unless Tester::D();
}

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();
